package com.usky.fire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.fire.domain.AlarmFire;
import com.usky.fire.mapper.AlarmFireMapper;
import com.usky.fire.service.AlarmFireService;
import com.usky.fire.service.util.OnlineMethod;
import com.usky.fire.service.vo.AlarmFireStatisticalVo;
import com.usky.fire.service.vo.AlarmFireVo;
import com.usky.system.model.LoginUser;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author JCB
 * @since 2022-08-18
 */
@Service
public class AlarmFireServiceImpl extends AbstractCrudService<AlarmFireMapper, AlarmFire> implements AlarmFireService {

    @Override
    public CommonPage<AlarmFireVo> alarmFireLowerList(String companyName, Integer handlingStatus, String startDate, String endDate,
                                                      Integer pageNum, Integer pageSize, String alarmType) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        List<String> alarmTypeList = new ArrayList<>();
        if (StringUtils.isNotBlank(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(alarmTypeArray[i]);
            }
        }
        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少单位ID部分代码
        List<Integer> companyIdList = new ArrayList<>();
        CommonPage<AlarmFire> alarmFireList = this.alarmFireList(companyIdList, companyName, handlingStatus, startDate, endDate, pageNum, pageSize, alarmTypeList);
        List<AlarmFireVo> list = new ArrayList<>();
        for (int i = 0; i < alarmFireList.getRecords().size(); i++) {
            AlarmFireVo alarmFireVo = new AlarmFireVo();
            alarmFireVo.setId(alarmFireList.getRecords().get(i).getId());
            alarmFireVo.setDeviceCode(alarmFireList.getRecords().get(i).getDeviceCode());
            alarmFireVo.setCompanyName(alarmFireList.getRecords().get(i).getCompanyName());
            alarmFireVo.setCompanyId(alarmFireList.getRecords().get(i).getCompanyId());
            alarmFireVo.setAlarmTime(df.format(alarmFireList.getRecords().get(i).getAlarmTime()));
            alarmFireVo.setAlarmType(alarmFireList.getRecords().get(i).getAlarmType());
            alarmFireVo.setConten(alarmFireList.getRecords().get(i).getDeviceName() + "," + alarmFireList.getRecords().get(i).getAttributeValue4());
            alarmFireVo.setHandlingStatus(alarmFireList.getRecords().get(i).getHandlingStatus());
            alarmFireVo.setHandler(alarmFireList.getRecords().get(i).getHandler());
            if (alarmFireList.getRecords().get(i).getHandlingTime() != null) {
                alarmFireVo.setHandlingTime(df.format(alarmFireList.getRecords().get(i).getHandlingTime()));
            } else {
                alarmFireVo.setHandlingTime(null);
            }
            alarmFireVo.setFalseAlarm(alarmFireList.getRecords().get(i).getFalseAlarm());
            alarmFireVo.setHandlingContent(alarmFireList.getRecords().get(i).getHandlingContent());
            alarmFireVo.setSitePhoto(alarmFireList.getRecords().get(i).getSitePhoto());
            alarmFireVo.setPortNum(alarmFireList.getRecords().get(i).getPortNum());
            alarmFireVo.setHostNum(alarmFireList.getRecords().get(i).getHostNum());
            alarmFireVo.setLoopNum(alarmFireList.getRecords().get(i).getLoopNum());
            alarmFireVo.setPtNum(alarmFireList.getRecords().get(i).getPtNum());
//            alarmFireVo.setSystemType(alarmFireList.getRecords().get(i).getSystemType());
//            alarmFireVo.setAddress(alarmFireList.getRecords().get(i).getAddress());
//            alarmFireVo.setAlarmStatus(alarmFireList.getRecords().get(i).getAlarmStatus());
//            alarmFireVo.setPointType(alarmFireList.getRecords().get(i).getPointType());
//            alarmFireVo.setHandlerPhone(alarmFireList.getRecords().get(i).getHandlerPhone());
            list.add(alarmFireVo);
        }
        return new CommonPage<>(list, alarmFireList.getTotal(), pageSize, pageNum);
    }


    @Override
    public Map<String, Object> fireStatistics(String alarmType) {
        List<String> alarmTypeList = new ArrayList<>();
        if (StringUtils.isNotBlank(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(alarmTypeArray[i]);
            }
        }

        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少维保单位部分代码
        List<Integer> companyIdList = new ArrayList<>();

        int untreated = this.fireCount(companyIdList, 0, alarmTypeList);
        int processed = this.fireCount(companyIdList, 1, alarmTypeList);
//        String disposalRate =  processed / (untreated + processed) + "%";
        Integer count = untreated + processed;
        String disposalRate = OnlineMethod.myPercent(processed, count);
        Map<String, Object> map = new HashMap<>();
        map.put("untreated", untreated);//告警未处理数
        map.put("processed", processed);//告警未处理数
        map.put("disposalRate", disposalRate);//告警处理率
        map.put("alarmCount", count);//告警总数
//        map.put("deviceCount", 0);//设备总数
//        map.put("deviceOfflineCount", 0);//设备离线数
//        map.put("deviceAlarmCount", 0);//设备告警数
//        map.put("deviceNormalCount", 0);//设备正常数
//        map.put("deviceOfflineRate", 0);//设备离线率
//        map.put("deviceAlarmRate", 0);//设备告警率
//        map.put("deviceOfflineRate", 0);//设备正常率
        return map;
    }

    @Override
    public Map<String, Object> fireStatisticalChart(String alarmType, String startDate, String endDate) {
        List<String> alarmTypeList = new ArrayList<>();
        if (StringUtils.isNotBlank(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(alarmTypeArray[i]);
            }
        }

        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少维保单位部分代码
        List<Integer> companyIdList = new ArrayList<>();
        List<String> alarmTypeListOne = new ArrayList<>();
        Integer selectStatus = 0;
        if (alarmTypeList.contains("100")) {
            selectStatus = 1;
            alarmTypeListOne.add("2");
            alarmTypeListOne.add("4");
        } else {
            alarmTypeListOne = alarmTypeList;
        }
        List<AlarmFireStatisticalVo> list = baseMapper.selectAlarmFireStatistical(companyIdList, alarmTypeListOne, startDate, endDate, selectStatus);
        List<Integer> alarmCountList = new ArrayList<>();
        List<Integer> handleCountList = new ArrayList<>();
        List<String> alarmTimeList = new ArrayList<>();
        List<String> handleRateList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            alarmCountList.add(list.get(i).getCount());
            handleCountList.add(list.get(i).getSum());
            alarmTimeList.add(list.get(i).getAlarmTime());
            handleRateList.add(OnlineMethod.myPercent(list.get(i).getSum(), list.get(i).getCount()));
        }
        Map<String, Object> map = new HashMap<>();
        map.put("alarmCountList", alarmCountList);
        map.put("handleCountList", handleCountList);
        map.put("alarmTimeList", alarmTimeList);
        map.put("handleRateList", handleRateList);
        return map;
    }


    @Override
    public void updateAlarmFire(AlarmFire alarmFire) {
        if (alarmFire.getId() == 0) {
            LambdaQueryWrapper<AlarmFire> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.select(AlarmFire::getId)
                    .eq(AlarmFire::getDeviceCode, alarmFire.getDeviceCode())
                    .eq(AlarmFire::getAlarmType, alarmFire.getAlarmType())
                    .eq(AlarmFire::getHostNum, alarmFire.getHostNum())
                    .eq(AlarmFire::getLoopNum, alarmFire.getLoopNum())
                    .eq(AlarmFire::getPtNum, alarmFire.getPtNum())
                    .eq(AlarmFire::getPortNum, alarmFire.getPortNum());
            List<AlarmFire> list = this.list(queryWrapper);
            if (CollectionUtils.isNotEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    list.get(i).setHandler(SecurityUtils.getUsername());
                    list.get(i).setHandlingTime(LocalDateTime.now());
                    list.get(i).setHandlingStatus(1);
                    list.get(i).setHandlingContent(alarmFire.getHandlingContent());
                    list.get(i).setHandlerPhone("未定义");
                    list.get(i).setFalseAlarm(alarmFire.getFalseAlarm());
                    this.updateById(list.get(i));
                }
            }
        } else {
            alarmFire.setHandler(SecurityUtils.getUsername());
            alarmFire.setHandlingTime(LocalDateTime.now());
            alarmFire.setHandlingStatus(1);
            this.updateById(alarmFire);
        }
    }


    @Override
    public List<AlarmFireVo> alarmFireLowerListExport(String companyName, Integer handlingStatus, String startDate, String endDate, String alarmType) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        List<String> alarmTypeList = new ArrayList<>();
        if (StringUtils.isNotBlank(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(alarmTypeArray[i]);
            }
        }
        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少单位ID部分代码
        List<Integer> companyIdList = new ArrayList<>();
        CommonPage<AlarmFire> alarmFireList = this.alarmFireList(companyIdList, companyName, handlingStatus, startDate, endDate, 0, 0, alarmTypeList);
        List<AlarmFireVo> list = new ArrayList<>();
        for (int i = 0; i < alarmFireList.getRecords().size(); i++) {
            AlarmFireVo alarmFireVo = new AlarmFireVo();
            alarmFireVo.setId(alarmFireList.getRecords().get(i).getId());
            alarmFireVo.setDeviceCode(alarmFireList.getRecords().get(i).getDeviceCode());
            alarmFireVo.setCompanyName(alarmFireList.getRecords().get(i).getCompanyName());
            alarmFireVo.setCompanyId(alarmFireList.getRecords().get(i).getCompanyId());
            alarmFireVo.setAlarmTime(df.format(alarmFireList.getRecords().get(i).getAlarmTime()));
            alarmFireVo.setAlarmType(alarmFireList.getRecords().get(i).getAlarmType());
            alarmFireVo.setConten(alarmFireList.getRecords().get(i).getDeviceName() + "," + alarmFireList.getRecords().get(i).getAttributeValue4());
            alarmFireVo.setHandlingStatus(alarmFireList.getRecords().get(i).getHandlingStatus());
            alarmFireVo.setHandler(alarmFireList.getRecords().get(i).getHandler());
            if (alarmFireList.getRecords().get(i).getHandlingTime() != null) {
                alarmFireVo.setHandlingTime(df.format(alarmFireList.getRecords().get(i).getHandlingTime()));
            } else {
                alarmFireVo.setHandlingTime(null);
            }
            alarmFireVo.setFalseAlarm(alarmFireList.getRecords().get(i).getFalseAlarm());
            alarmFireVo.setHandlingContent(alarmFireList.getRecords().get(i).getHandlingContent());
//            alarmFireVo.setSystemType(alarmFireList.getRecords().get(i).getSystemType());
//            alarmFireVo.setAddress(alarmFireList.getRecords().get(i).getAddress());
//            alarmFireVo.setAlarmStatus(alarmFireList.getRecords().get(i).getAlarmStatus());
//            alarmFireVo.setPointType(alarmFireList.getRecords().get(i).getPointType());
//            alarmFireVo.setPortNum(alarmFireList.getRecords().get(i).getPortNum());
//            alarmFireVo.setHostNum(alarmFireList.getRecords().get(i).getHostNum());
//            alarmFireVo.setLoopNum(alarmFireList.getRecords().get(i).getLoopNum());
//            alarmFireVo.setPtNum(alarmFireList.getRecords().get(i).getPtNum());
//            alarmFireVo.setHandlerPhone(alarmFireList.getRecords().get(i).getHandlerPhone());
//            alarmFireVo.setSitePhoto(alarmFireList.getRecords().get(i).getSitePhoto());
            list.add(alarmFireVo);
        }
        return list;
    }

    @Override
    public Integer fireCount(List<Integer> companyIdList, Integer handlingStatus, List<String> alarmTypeList) {
        LambdaQueryWrapper<AlarmFire> queryWrapper = Wrappers.lambdaQuery();
        if (companyIdList.size() > 0) {
            queryWrapper.in(AlarmFire::getCompanyId, companyIdList);
        }
        if (handlingStatus != null) {
            queryWrapper.eq(AlarmFire::getHandlingStatus, handlingStatus);
        }
        if (alarmTypeList.size() > 0) {
            if (alarmTypeList.contains("100")) {
                List<String> alarmTypeList1 = new ArrayList<>();
                alarmTypeList1.add("2");
                alarmTypeList1.add("4");
                queryWrapper.notIn(AlarmFire::getAlarmType, alarmTypeList1);
            } else {
                queryWrapper.in(AlarmFire::getAlarmType, alarmTypeList);
            }
        }
        int total = this.count(queryWrapper);
        return total;
    }


    @Override
    public CommonPage<AlarmFire> alarmFireList(List<Integer> companyIdList, String companyName, Integer handlingStatus, String startDate,
                                               String endDate, Integer pageNum, Integer pageSize, List<String> alarmTypeList) {
        LambdaQueryWrapper<AlarmFire> queryWrapper = Wrappers.lambdaQuery();
        if (companyIdList.size() > 0) {
            queryWrapper.in(AlarmFire::getCompanyId, companyIdList);
        }
        if (StringUtils.isNotBlank(companyName)) {
            queryWrapper.like(AlarmFire::getCompanyName, companyName);
        }
        if (handlingStatus != null) {
            queryWrapper.eq(AlarmFire::getHandlingStatus, handlingStatus);
        }

        if (alarmTypeList.size() > 0) {
            if (alarmTypeList.contains("100")) {
                List<String> alarmTypeList1 = new ArrayList<>();
                alarmTypeList1.add("2");
                alarmTypeList1.add("4");
                queryWrapper.notIn(AlarmFire::getAlarmType, alarmTypeList1);
            } else {
                queryWrapper.in(AlarmFire::getAlarmType, alarmTypeList);
            }
        }

        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            queryWrapper.between(AlarmFire::getAlarmTime, startDate, endDate);
        }
        int total = this.count(queryWrapper);
        queryWrapper.orderByDesc(AlarmFire::getId);
        if (pageNum != null && pageNum != 0 && pageSize != null && pageSize != 0) {
            Integer startFate = OnlineMethod.getStartFate(pageNum, pageSize);
            queryWrapper.last("limit " + startFate + "," + pageSize);
        }
        List<AlarmFire> list = this.list(queryWrapper);
        return new CommonPage<>(list, total, pageSize, pageNum);
    }
}
